import netCDF4
from pmc_analysis.lib.pmc_analysis_app import pmc_analysis_app

from pmc_analysis.lib.tools import blosc_file as blosc
from pmc_analysis.lib.image_processing import flat_field as ff
from pmc_analysis.lib.image_processing import flat_field_piggyback as ffpb
from pmc_analysis.lib.image_processing import binning
from pmc_analysis.lib.tools import periodic_image_finder as pif
from pmc_analysis.lib.tools import GPS_pointing_and_sun as GPS

import numpy as np
from datetime import datetime
import os


def make_datetime_from_timestamp(timestamp):
    DT = datetime.utcfromtimestamp(timestamp).strftime('%Y-%m-%dT%H:%M:%S')
    return DT


class NetCDFConversionApp(pmc_analysis_app.App):
    def create_netcdf(self):
        self.create_output()
        self.logger.info('Creating container classes')
        bin_size = self.settings.bin_size
        Times = np.arange(self.settings.start, self.settings.stop, self.settings.interval)
        N = len(Times)
        n = len(self.settings.camera_numbers)
        Img_data = np.zeros((N, n, int(3232/bin_size), int(4864/bin_size)))
        Lats = np.zeros(N)
        Lons = np.zeros(N)
        Alts = np.zeros(N)
        Azs = np.zeros(N)
        Exposures = np.zeros((N,n))

        for j in range(n):
            fns = pif.find_periodic_images_full_fn(self.settings.camera_numbers[j], self.settings.start, self.settings.stop, self.settings.interval)
            for i in range(N):
                fn = fns[i]
                ts = int(fn.split('=')[-1][:10])
                if self.settings.piggyback:
                    if bin_size > 1:
                        img_data = binning.bucket(ffpb.get_final_cleaned_image(fn, self.settings.new_flat_field_window), (bin_size, bin_size))
                    else:
                        img_data = ffpb.get_final_cleaned_image(fn, 600)
                else:
                    if bin_size > 1:
                        img_data = binning.bucket(ff.get_final_cleaned_image(fn, self.settings.new_flat_field_window), (bin_size, bin_size))
                    else:
                        img_data = ff.get_final_cleaned_image(fn, 600)
                _, chunk = blosc.load_blosc_image(fn)
                lat_value, lon_value, alt_value = GPS.get_lat_long_alt_SIP(ts)
                az_value = GPS.get_pointing_rotator(ts)
                Img_data[i,j,:,:] = img_data
                if j == 0:
                    Lats[i] = lat_value
                    Lons[i] = lon_value
                    Alts[i] = alt_value/3280.
                    Azs[i] = az_value
                Exposures[i,j] = chunk[0][6]
                print(ts)

        outpath = os.path.join(self.out_path, self.settings.netcdf_filename)
        dataset = netCDF4.Dataset(outpath, "w", format="NETCDF3_CLASSIC")

        dataset.createDimension('x_pixel', int(3232/bin_size))
        dataset.createDimension('y_pixel', int(4864/bin_size))
        dataset.createDimension('Times', None)
        dataset.createDimension('Camera_numbers', n)

        Imgs = dataset.createVariable('Imgs', 'i2', ('Times', 'Camera_numbers', 'x_pixel', 'y_pixel'))
        Time = dataset.createVariable('Time', 'f8', ('Times'))
        Az = dataset.createVariable('Azimuth', 'f4', ('Times'))
        Exposure = dataset.createVariable('Exposure', 'f4', ('Times', 'Camera_numbers'))
        Camera_number = dataset.createVariable('Camera_numbers', 'i2', ('Camera_numbers'))
        X_pixel = dataset.createVariable('x_pixel_index', 'i2', ('x_pixel'))
        Y_pixel = dataset.createVariable('y_pixel_index', 'i2', ('y_pixel'))

        Imgs.UNITS = 'Uncalibrated brightness units'
        Imgs.CATDESC = 'PMC Turbo optical images. Taken with Kodak 16070 CCD and Hoya #25A red filter.'
        Imgs.DISPLAY_TYPE = 'image'
        Imgs.FIELDNAM = 'Images'
        Imgs.VAR_TYPE = 'data'
        Imgs.DEPEND_0 = 'Time'
        Imgs.DEPEND_1 = 'Camera_numbers'
        Imgs.DEPEND_2 = 'x_pixel_index'
        Imgs.DEPEND_3 = 'y_pixel_index'
        Imgs.FILLVAL = np.array([-32768], dtype=np.int16)[0]
        Imgs.VALIDMIN = np.array([-32767], dtype=np.int16)[0]
        Imgs.VALIDMAX = np.array([32767], dtype=np.int16)[0]
        Imgs.FORMAT ='I6'
        Imgs.LABL_PTR_1 = 'Height'
        Imgs.LABL_PTR_2 = 'Width'
            

        Time.UNITS = 'Seconds since 1970-01-01T00:00:00Z'
        Time.CATDESC = 'Unix time - seconds since 1970-01-01 00:00:00'
        Time.DISPLAY_TYPE = 'no_plot'
        Time.FIELDNAM = 'Timestamp'
        Time.VAR_TYPE = 'support_data'
        Time.FILLVAL = np.array([-1.0E31], dtype=np.float32)[0]
        Time.VALIDMIN = np.array([0.], dtype=np.float32)[0]
        Time.VALIDMAx = np.array([1.0E31], dtype=np.float32)[0]
        Time.FORMAT ='F12.2'
        Time.LABLAXIS = 'Time'
        

        Az.UNITS = 'Radians'
        Az.CATDESC = 'Radians CW from north (as viewed from above)'
        Az.DISPLAY_TYPE = 'time_series'
        Az.FIELDNAM = 'Azimuth'
        Az.VAR_TYPE = 'data'
        Az.DEPEND_0 = 'Time'
        Az.FILLVAL = np.array([-1.0E31], dtype=np.float32)[0]
        Az.VALIDMIN = np.array([0.], dtype=np.float32)[0]
        Az.VALIDMAX = np.array([6.283185], dtype=np.float32)[0]
        Az.FORMAT ='F5.2'
        Az.LABLAXIS = 'Azimuth'

        Exposure.UNITS = 'microseconds'
        Exposure.CATDESC = 'Exposure time of image'
        Exposure.DISPLAY_TYPE = 'time_series'
        Exposure.FIELDNAM = 'Exposure Time'
        Exposure.VAR_TYPE = 'data'
        Exposure.DEPEND_0 = 'Time'
        Exposure.DEPEND_1 = 'Camera_numbers'
        Exposure.FILLVAL = np.array([-1.0E31], dtype=np.float32)[0]
        Exposure.VALIDMIN = np.array([0.], dtype=np.float32)[0]
        Exposure.VALIDMAX = np.array([2.e7], dtype=np.float32)[0]
        Az.FORMAT = 'F7.0'
        Az.LABLAXIS = 'Exposure Time'

        Camera_number.UNITS = ' '
        Camera_number.CATDESC = ('Cameras numbered 1 to 3 right-to-left (narrow FOV)' +
                               ' and 4 to 7 left-to-right (wide FOV). See Fritts 2019' +
                               'overview paper for more details')
        Camera_number.DISPLAY_TYPE = 'no_plot'
        Camera_number.FIELDNAM = 'Camera numbers'
        Camera_number.VAR_TYPE = 'support_data'
        Camera_number.FILLVAL = np.array([-32768], dtype=np.int16)[0]
        Camera_number.VALIDMIN = np.array([1], dtype=np.int16)[0]
        Camera_number.VALIDMAX = np.array([99], dtype=np.int16)[0]
        Camera_number.FORMAT ='I2'
        Camera_number.LABLAXIS = 'Camera Number'


        X_pixel.UNITS = ' '
        X_pixel.CATDESC = ('pixel indices x axis')
        X_pixel.DISPLAY_TYPE = 'no_plot'
        X_pixel.FIELDNAM = 'x pixel'
        X_pixel.VAR_TYPE = 'support_data'
        X_pixel.FILLVAL = np.array([-32768], dtype=np.int16)[0]
        X_pixel.VALIDMIN = np.array([1], dtype=np.int16)[0]
        X_pixel.VALIDMAX = np.array([int(3232/bin_size)], dtype=np.int16)[0]
        X_pixel.FORMAT ='I2'
        X_pixel.LABLAXIS = 'X Pixel'


        Y_pixel.UNITS = ' '
        Y_pixel.CATDESC = ('pixel indices y axis')
        Y_pixel.DISPLAY_TYPE = 'no_plot'
        Y_pixel.FIELDNAM = 'y pixel'
        Y_pixel.VAR_TYPE = 'support_data'
        Y_pixel.FILLVAL = np.array([-32768], dtype=np.int16)[0]
        Y_pixel.VALIDMIN = np.array([1], dtype=np.int16)[0]
        Y_pixel.VALIDMAX = np.array([int(4864/bin_size)], dtype=np.int16)[0]
        Y_pixel.FORMAT ='I2'
        Y_pixel.LABLAXIS = 'Y Pixel'

        dataset.Source_name = netCDF4.chartostring(netCDF4.stringtochar(np.array('PMC Turbo>Polar Mesospheric Cloud Turbulence', dtype='S')))
        dataset.Data_type = netCDF4.chartostring(netCDF4.stringtochar(np.array('L1>Corrected Binned Images', dtype='S')))
        dataset.Descriptor = netCDF4.chartostring(netCDF4.stringtochar(np.array('PMC Cameras', dtype='S')))
        dataset.Data_version = netCDF4.chartostring(netCDF4.stringtochar(np.array('1', dtype='S')))
        dataset.Logical_file_id = netCDF4.chartostring(netCDF4.stringtochar(np.array('pmc_turbo_l1_images', dtype='S')))
        dataset.PI_name = netCDF4.chartostring(netCDF4.stringtochar(np.array('David Fritts', dtype='S')))
        dataset.PI_affiliation = netCDF4.chartostring(netCDF4.stringtochar(np.array('GATS inc', dtype='S')))
        dataset.TEXT = netCDF4.chartostring(netCDF4.stringtochar(np.array('', dtype='S')))
        dataset.Instrument_type = netCDF4.chartostring(netCDF4.stringtochar(np.array('Imaging and Remote Sensing (ITM/Earth)', dtype='S')))
        dataset.Discipline = netCDF4.chartostring(netCDF4.stringtochar(np.array('Heliophysics>Mesosphere and Thermosphere Dynamics', dtype='S')))
        dataset.Logical_source = netCDF4.chartostring(netCDF4.stringtochar(np.array('pmc-turbo_l1_corrected_binned_images', dtype='S')))
        dataset.Logical_source_description = netCDF4.chartostring(netCDF4.stringtochar(np.array('PMC Turbo corrected and binned images', dtype='S')))
        dataset.Acknowledgement = netCDF4.chartostring(netCDF4.stringtochar(np.array('', dtype='S')))
        dataset.LINK_TEXT = netCDF4.chartostring(netCDF4.stringtochar(np.array('PMC Turbo experiment described in the', dtype='S')))
        dataset.LINK_TITLE = netCDF4.chartostring(netCDF4.stringtochar(np.array('PMC Turbo overview paper', dtype='S')))
        dataset.HTTP_Link = netCDF4.chartostring(netCDF4.stringtochar(np.array('https://doi.org/10.1029/2019JD030298', dtype='S')))
        dataset.Mission_group = netCDF4.chartostring(netCDF4.stringtochar(np.array('Long Distance Balloon missions', dtype='S')))

        dataset.summary = netCDF4.chartostring(netCDF4.stringtochar(np.array('PMC-Turbo images, flatfielded, with 10-minute moving window subtracted. Downsampled with 4x4 pixel binning', dtype='S')))

        X_pixel[:] = np.arange(1, int(3232/bin_size)+1, 1)
        Y_pixel[:] = np.arange(1, int(4864/bin_size)+1, 1)
        Imgs[:,:,:,:] = Img_data.astype(np.int16)
        Time[:] = Times
        Az[:] = Azs
        Exposure[:,:] = Exposures
        Camera_number[:] = self.settings.camera_numbers
        dataset.close()
        return


if __name__ == "__main__":
    import warnings; warnings.simplefilter('ignore')
    app =  NetCDFConversionApp()
    app.create_netcdf()



import numpy as np

from skywinder_analysis.lib.tools import blosc_file
from skywinder_analysis.lib.image_processing import flat_field
from collections import deque


class RollingFlatField():
    def __init__(self, size=10):
        self.images_for_flat_field = deque(maxlen=size)
        self.size = size

    def roll(self, image_filename):
        if len(self.images_for_flat_field) == self.size:
            popped = self.images_for_flat_field.popleft()
        image, _ = blosc_file.load_blosc_image(image_filename)
        self.images_for_flat_field.append(image)

    def generate_flat_field(self):
        return flat_field.generate_flat_field_image_from_raw(self.images_for_flat_field)



from default_settings import settings
import numpy as np
import matplotlib.animation as animation
import matplotlib.cm as cm


import os
from pmc_analysis.lib.tools import periodic_image_finder
#n = 1531402200 # 2018-07-12_1330
#start = n - (70*60)
#stop = start + (60*60)
#interval = 10

#n = 1531054800 # 2018-07-08_1300
n = 1531098000 # 2018-07-09_0100
start = n
stop = start + (120*60)
interval = 10

settings.camera_numbers = [4,5,6,7]
settings.all_filename_lists = {4: [], 5:[], 6:[], 7:[]}
for cam_num in settings.camera_numbers:
    filenames = periodic_image_finder.find_periodic_images_full_fn(cam_num,
                                                                            start_timestamp=start,
                                                                            stop_timestamp=stop,
                                                                            interval=interval)

    settings.all_filename_lists[cam_num] = filenames

settings.min_x = -100
settings.max_x = 100
settings.x_increment = 2000

settings.min_y = 10
settings.max_y = 110
settings.y_increment = 1000

settings.bin_factor = 8

settings.xvalues = np.linspace(settings.min_x, settings.max_x, num=settings.x_increment)
settings.yvalues = np.linspace(settings.min_y, settings.max_y, num=settings.y_increment)
settings.individual_frames = True
settings.grid = False
settings.grid_spacing = 5
settings.color_scheme = cm.Greys_r

settings.timestamp_in_image = False



import numpy as np
import matplotlib.pyplot as plt
import cartopy
import cartopy.crs as ccrs


def plot_mollweide(ra_dec_coords, degrees=False):
    fig = plt.figure(figsize=(10, 5))
    ax = fig.add_subplot(111, projection="mollweide", axisbg='LightCyan')
    ax.grid(True)
    for coords in ra_dec_coords:
        fov = np.array(coords)
        ra = fov[:, 0]
        dec = fov[:, 1]
        if degrees:
            ra = np.deg2rad(ra)
            dec = np.deg2rad(dec)
        ax.plot(ra, dec, '.')


def plot_az_alt_cartesian(all_altaz_coords):
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.grid(True)
    ax.set_ylabel('Alt (deg)')
    ax.set_xlabel('Az (deg)')
    for coords in all_altaz_coords[:]:
        alts = []
        azs = []
        for coord in coords:
            alts.append(coord.alt.value)
            azs.append(coord.az.value)
        ax.plot(azs, alts, '.')
    ax.plot()
    ax.set_ylim(15, 70)


def plot_orthographic_projection(all_altaz_coords, central_longitude, central_latitude, az_offset):
    plt.figure(figsize=(9, 9))
    ax = plt.axes(projection=ccrs.Orthographic(central_longitude, central_latitude))

    ax.set_global()
    ax.gridlines()

    colors = 'bgrcmyk'

    ax.plot(range(-180, 180, 1), [65] * len(range(-180, 180, 1)), color='black', linewidth=1, marker='.',
            transform=ccrs.Geodetic())
    # 25 degrees off zenith

    for i, corners in enumerate(all_altaz_coords):
        lats = []
        lons = []
        for corner in corners:
            lats.append(corner.alt.value)
            lons.append(corner.az.value - az_offset)

        ax.scatter(lons, lats,
                   color=colors[i], marker='o', transform=ccrs.Geodetic())


def plot_gnomonic_projection(all_altaz_coords, az_offset):
    plt.figure(figsize=(9, 9))
    ax = plt.axes(projection=ccrs.Gnomonic(central_latitude=90))

    ax.set_global()
    ax.gridlines()

    colors = 'bgrcmyk'

    ax.plot(range(-180, 180, 1), [65] * len(range(-180, 180, 1)), color='black', linewidth=1, marker='.',
            transform=ccrs.Geodetic())
    # 25 degrees off zenith

    for i, coords in enumerate(all_altaz_coords):
        lats = []
        lons = []
        for coord in coords:
            lats.append(coord.alt.value)
            lons.append(coord.az.value - az_offset)

        ax.scatter(lons, lats,
                   color=colors[i], marker='o', transform=ccrs.Geodetic())



from default_settings import settings
import matplotlib.animation as animation
import glob
import os

settings.camera_number = 3

fns = glob.glob('/data/mounted_filesystems/nas2/10_minutes_piggyback/*')
print(fns)

def timestamp(x):
    return x[63:69]

sorted_fns = sorted(fns, key = timestamp)

settings.filenames = sorted_fns

settings.flat_field_filenames = settings.filenames[::10]
settings.writer = animation.writers['ffmpeg'](fps=20, codec='mpeg4', bitrate=2**20)
settings.bin_pixels = (4,4)
settings.rotate = False
#settings.section = [0,3232,0,4864]
#settings.section = [0,3232,0,(5*4864)//8]



from skywinder_analysis.lib.tools import generic
import numpy as np
from skywinder_analysis.lib.image_processing import flat_field
import matplotlib.animation as animation
import matplotlib.cm as cm

settings = generic.Class()

settings.writer = animation.writers['ffmpeg'](fps=10, codec='h264', bitrate=2 ** 20)
settings.cam_nums = [4, 5, 6, 7]
settings.bin_factor = 4
settings.dpi = 100
settings.saved_flat_field_path = None
settings.all_flat_field_filenames = None
settings.new_flat_field_window = 600

settings.relevel_images = True

settings.pointing_directory = ''

settings.output_name = 'image.png'
settings.all_filenames = {}  # use glob to make list of filenames
settings.min_percentile = 0.1
settings.max_percentile = 99.9

settings.min_x = -100
settings.max_x = 100
settings.x_increment = 100

settings.min_y = 0
settings.max_y = 115
settings.y_increment = 100

settings.xvalues = np.linspace(settings.min_x, settings.max_x, num=settings.x_increment)
settings.yvalues = np.linspace(settings.min_y, settings.max_y, num=settings.y_increment)

settings.provided_grid_map = False
settings.level_region = [0 // settings.bin_factor,
                         3232 // settings.bin_factor,
                         4864 // (4 * settings.bin_factor),
                         (3*4864) // (4*settings.bin_factor)]

settings.individual_frames = False
settings.grid = False
settings.grid_spacing = 25

settings.color_scheme = cm.inferno
settings.timestamp_in_image = True
settings.text_color = 'black'
settings.use_new_flat_fields = True
settings.flat_field_filenames = False
settings.wireframes = False



from default_settings import settings
import matplotlib.animation as animation


import os
from pmc_analysis.lib.tools import periodic_image_finder
n = 1531400400 # 2018-07-12_1300
start = n + (31*60) + 38
#end = start + (14*60)
end = start + (5*60)
interval = 2

settings.camera_number = 5

filenames = periodic_image_finder.find_periodic_images(settings.camera_number, start, end, interval)

joined_filenames = []
for fn in filenames:
        joined_filenames.append(os.path.join(('/data/mounted_filesystems/nas2/c%d/'%settings.camera_number), fn))
settings.filenames = joined_filenames

settings.flat_field_filenames = settings.filenames#[::5]
settings.writer = animation.writers['ffmpeg'](fps=20, codec='mpeg4', bitrate=2**20)
settings.rotate = False
settings.section_x = 4864-500
settings.section_y = 1300-500
settings.section_xwidth = 500
settings.section_ywidth = 500
settings.xshift = -14
settings.yshift = -4
settings.output_name = 'c5_2018-07-12_feature_and_context.mp4'



from default_settings import settings
import matplotlib.cm as cm
import numpy as np
import matplotlib.animation as animation


import os
from pmc_analysis.lib.tools import periodic_image_finder

n = 1531402200 # 2018-07-12_1330
start = n - (3*60)
stop = start + (7*60)
interval = 2

settings.camera_numbers = [4,5,6,7]
settings.all_filename_lists = {4: [], 5:[], 6:[], 7:[]}
for cam_num in settings.camera_numbers:
    filenames = periodic_image_finder.find_periodic_images_full_fn(cam_num,
                                                                            start_timestamp=start,
                                                                            stop_timestamp=stop,
                                                                            interval=interval)

    settings.all_filename_lists[cam_num] = filenames

#settings.min_x = -100
#settings.max_x = 100
#settings.x_increment = 2000

#settings.min_y = 10
#settings.max_y = 110
#settings.y_increment = 1000

#settings.bin_factor = 8

#settings.min_x = -70
#settings.max_x = -15
#settings.x_increment = 1000

#settings.min_y = 15
#settings.max_y = 70
#settings.y_increment = 1000
 
settings.min_x = -40
settings.max_x = -20
settings.x_increment = 1000

settings.min_y = 25
settings.max_y = 45
settings.y_increment = 1000

#settings.min_x = -70
#settings.max_x = -45
#settings.x_increment = 1000

#settings.min_y = 25
#settings.max_y = 50
#settings.y_increment = 1000

settings.bin_factor = 2

settings.xvalues = np.linspace(settings.min_x, settings.max_x, num=settings.x_increment)
settings.yvalues = np.linspace(settings.min_y, settings.max_y, num=settings.y_increment)
settings.individual_frames = False
settings.grid = True
settings.grid_spacing = 5

#settings.provided_grip_map = '/data/mounted_filesystems/nas2/resources/grid_mapping_lookup_tables/1000x1000_khi_section.csv'

#settings.color_scheme = cm.Blues_r



import os
import numpy as np
from matplotlib import pyplot as plt
import matplotlib.cm as cm
import scipy.stats as ss
from scipy.interpolate import RectBivariateSpline as RBS
from skywinder_analysis.lib.image_processing import flat_field, binning
from skywinder_analysis.lib.tools import GPS_pointing_and_sun as GPS
from skywinder_analysis.lib.tools import periodic_image_finder
from astropy.coordinates import EarthLocation
from astropy import units as u

def get_image_translation_Earth_frame(timestamp0, timestamp1):
    #Gets the translation of the gondola from one timestamp to the next in the coordinate system defined
    #at timestamp0 (with +y axis pointing towards North and +x axis pointing East)
    lat0, long0, alt0 = GPS.get_lat_long_alt_SIP(timestamp0)
    lat1, long1, alt1 = GPS.get_lat_long_alt_SIP(timestamp1)
    mylocation0 = EarthLocation(lat=lat0*u.deg,  lon=long0*u.deg, height=alt0/3.28*u.meter)
    mylocation1 = EarthLocation(lat=lat1*u.deg,  lon=long1*u.deg, height=alt1/3.28*u.meter)
    x1 = np.array([mylocation1.x.value - mylocation0.x.value,
               mylocation1.y.value - mylocation0.y.value, 
               mylocation1.z.value - mylocation0.z.value])/1000.0
    # x1 is the translation vector between the gondola position at timestamp0 and timestamp1 in the 
    #geodetic frame (+x points to lat = 0, long = 0, +y points to lat = 0, long = 90 degrees,
    #+z points to lat = 90 degrees)
    
    theta = np.radians(long0)
    phi = np.radians(lat0)
    NE_x = np.array([-np.sin(theta),
                     np.cos(theta),
                    0])
    NE_y = np.array([-np.cos(theta)*np.sin(phi),
                    -np.sin(theta)*np.sin(phi),
                    np.cos(phi)])
    NE_z = np.array([np.cos(theta)*np.cos(phi),
                    np.sin(theta)*np.cos(phi),
                    np.sin(phi)])
    #The NE_i vectors define the local NESW coordinate system relative to the geodetic frame.
    
    x1_NE = np.array([np.dot(x1, NE_x), np.dot(x1, NE_y), np.dot(x1, NE_z)])
    #x1_G is the translation vector in the local NESW frame.
    
    return x1_NE

def get_image_translation_Earth_frame_array(timestamp0, timestamp_array):
    #Gets the translation of the gondola from one timestamp to the next in the coordinate system defined
    #at timestamp0 (with +y axis pointing towards North and +x axis pointing East)
    lat0, long0, alt0 = GPS.get_lat_long_alt_SIP(timestamp0)
    lat_array, long_array, alt_array = GPS.get_lat_long_alt_SIP_array(timestamp_array)
    mylocation0 = EarthLocation(lat=lat0*u.deg,  lon=long0*u.deg, height=alt0/3.28*u.meter)
    mylocation_array = EarthLocation(lat=lat_array*u.deg,  lon=long_array*u.deg, height=alt_array/3.28*u.meter)
    x1 = np.array([mylocation_array.x.value - mylocation0.x.value,
                   mylocation_array.y.value - mylocation0.y.value, 
                   mylocation_array.z.value - mylocation0.z.value])/1000.0
    # x1 is the translation vector between the gondola position at timestamp0 and timestamp1 in the 
    #geodetic frame (+x points to lat = 0, long = 0, +y points to lat = 0, long = 90 degrees,
    #+z points to lat = 90 degrees)
    
    theta = np.radians(long0)
    phi = np.radians(lat0)  
    NE_x = np.array([-np.sin(theta),
                     np.cos(theta),
                    0])
    NE_y = np.array([-np.cos(theta)*np.sin(phi),
                    -np.sin(theta)*np.sin(phi),
                    np.cos(phi)])
    NE_z = np.array([np.cos(theta)*np.cos(phi),
                    np.sin(theta)*np.cos(phi),
                    np.sin(phi)])
    #The NE_i vectors define the local NESW coordinate system relative to the geodetic frame.
    
    NE = np.array([NE_x, NE_y, NE_z])
    
    x1_NE = np.matmul(NE, x1)
    #x1_NE is the translation vector in the local NESW frame.
    
    return x1_NE

def get_x_y_arrays(camera_numbers=[4,5,6,7], bin_factor = 8, alignment='north', translate = False, current_time=1531350000,         reference_time=1531350000, clipped=False, projection='old'):
    #Gets X- and Y- arrays for scatter plot. alignment='north' returns arrays in coordinate system with +y to the north, +x to east.
    #alignment='antisun' returns arrays in coordinate system with +y in antisun direction.

    #In order to compensate for gondola translation, current time and reference time must be specified. The origin of the coordinate system will be 
    #directly overhead at the reference time - other images will be translated relative to that point. This only works for alignment='north'.

    x_old = np.arange(0, 3232, 34)
    y_old = np.arange(0, 4864, 34)

    if clipped:
        x_new = np.arange(0, 3232, bin_factor)
        y_new = np.arange(0, clipped, bin_factor)
    else:
        x_new = np.arange(0, 3232, bin_factor)
        y_new = np.arange(0, 4864, bin_factor)

    X_array = np.zeros(0)
    Y_array = np.zeros(0)
    
    for i in range(len(camera_numbers)):
        camera_number = camera_numbers[i]
        if projection == 'old':
            fn = '/data/mounted_filesystems/nas2/resources/pixel_coordinates_downsampled/c{0}_coordinates.csv'.format(camera_number)
        else:
            fn = '/data/mounted_filesystems/nas2/resources/pixel_coordinates_downsampled2/c{0}_coordinates.csv'.format(camera_number)
        XY = np.loadtxt(fn, delimiter=',')
        X = XY.T[0].reshape(96,144)
        Y = XY.T[1].reshape(96,144)
        f_x = RBS(x_old, y_old, X)
        X_inter = f_x(x_new, y_new)
        f_y = RBS(x_old, y_old, Y)
        Y_inter = f_y(x_new, y_new)
        
        X_array = np.hstack((X_array, X_inter.flatten()))
        Y_array = np.hstack((Y_array, Y_inter.flatten()))
    
    if translate and current_time == 1531350000 and reference_time == 1531350000:
        print('Warning: in order to account for gondola translation, current and reference time must be specified')
    if translate and alignment=='antisun':
        print('Warning: translate option not valid for antisun alignment')

    if alignment == 'north':
        az_offset = GPS.get_pointing_rotator(current_time)

        if translate:
            X_offset = get_image_translation_Earth_frame(reference_time, current_time)
        else:
            X_offset=np.array([0, 0, 0])

        scaling = 1.0 + X_offset[2]/45.0
        
        X_array_trans = (X_array*np.cos(az_offset)*scaling
                        + Y_array*np.sin(az_offset)*scaling
                        + X_offset[0])
        Y_array_trans = (-X_array*np.sin(az_offset)*scaling
                        + Y_array*np.cos(az_offset)*scaling
                        + X_offset[1])

    elif alignment == 'antisun':        
        X_array_trans = X_array
        Y_array_trans = Y_array
    else:
        print('alignment must be either \'north\' or \'antisun\'')
        return

    return X_array_trans, Y_array_trans

def convert_pixel_arrays_to_sky_coordinates(X_px, Y_px, timestamps, camera_number=5, bin_factor = 8, reference_time=1531350000,                       projection='old'):
    #Converts arrays of pixel coordinates to coordinates on the cloud layer. Returns arrays in coordinate system with +y to the north, +x to east.
    #In order to compensate for gondola translation, current time and reference time must be specified. The origin of the coordinate system will be 
    #directly overhead at the reference time - other images will be translated relative to that point. 

    x_old = np.arange(0, 3232, 34)
    y_old = np.arange(0, 4864, 34)
    if projection == 'old':
        fn = '/data/mounted_filesystems/nas2/resources/pixel_coordinates_downsampled/c{0}_coordinates.csv'.format(camera_number)
    else:
        fn = '/data/mounted_filesystems/nas2/resources/pixel_coordinates_downsampled2/c{0}_coordinates.csv'.format(camera_number)
    XY = np.loadtxt(fn, delimiter=',')
    X = XY.T[0].reshape(96,144)
    Y = XY.T[1].reshape(96,144)
    f_x = RBS(x_old, y_old, X)
    f_y = RBS(x_old, y_old, Y)
    X_array = np.zeros(0)
    Y_array = np.zeros(0)
    for x, y, timestamp in zip(X_px, Y_px, timestamps):
        X0 = f_x(x*bin_factor + bin_factor/2., y*bin_factor + bin_factor/2.).flatten()
        Y0 = f_y(x*bin_factor + bin_factor/2., y*bin_factor + bin_factor/2.).flatten()
    
        az_offset = GPS.get_pointing_rotator(timestamp)

        X_offset = get_image_translation_Earth_frame(reference_time, timestamp)
        
        scaling = 1.0 + X_offset[2]/45.0

        X0_trans = (X0*np.cos(az_offset)*scaling
                        + Y0*np.sin(az_offset)*scaling
                        + X_offset[0])
        Y0_trans = (-X0*np.sin(az_offset)*scaling
                        + Y0*np.cos(az_offset)*scaling
                        + X_offset[1])
        X_array = np.hstack((X_array, X0_trans))
        Y_array = np.hstack((Y_array, Y0_trans))    


    return X_array, Y_array

def convert_pixel_arrays_to_sky_coordinates_array(X_px, Y_px, timestamps, camera_number=5, bin_factor = 8, 
                                                 reference_time=1531350000, projection='old'):
    #Converts arrays of pixel coordinates to coordinates on the cloud layer. Returns arrays in coordinate system with +y to the north, +x to east.
    #In order to compensate for gondola translation, current time and reference time must be specified. The origin of the coordinate system will be 
    #directly overhead at the reference time - other images will be translated relative to that point. 

    x_old = np.arange(0, 3232, 34)
    y_old = np.arange(0, 4864, 34)
    if projection == 'old':
        fn = '/data/mounted_filesystems/nas2/resources/pixel_coordinates_downsampled/c{0}_coordinates.csv'.format(camera_number)
    else:
        fn = '/data/mounted_filesystems/nas2/resources/pixel_coordinates_downsampled2/c{0}_coordinates.csv'.format(camera_number)
    XY = np.loadtxt(fn, delimiter=',')
    X = XY.T[0].reshape(96,144)
    Y = XY.T[1].reshape(96,144)
    f_x = RBS(x_old, y_old, X)
    f_y = RBS(x_old, y_old, Y)
    X_array = np.zeros(0)
    Y_array = np.zeros(0)
    #for x, y, timestamp in zip(X_px, Y_px, timestamps):
        
    X0 = f_x.ev(X_px*bin_factor + bin_factor/2., Y_px*bin_factor + bin_factor/2.).flatten()
    Y0 = f_y.ev(X_px*bin_factor + bin_factor/2., Y_px*bin_factor + bin_factor/2.).flatten()
    
    az_offset = GPS.get_pointing_rotator_array(timestamps)
    X_offset = get_image_translation_Earth_frame_array(reference_time, timestamps)

    scaling = 1.0 + X_offset[2,:]/45.0

    X0_trans = (X0*np.cos(az_offset)*scaling
                    + Y0*np.sin(az_offset)*scaling
                    + X_offset[0,:])
    Y0_trans = (-X0*np.sin(az_offset)*scaling
                    + Y0*np.cos(az_offset)*scaling
                    + X_offset[1,:])

    return X0_trans, Y0_trans

def get_brightness_array(timestamp, camera_numbers=[4,5,6,7], bin_factor = 8, reflection_window = 600, clipped=False):
    #Returns brightness array for given time, cameras, etc. Output will have same format as X- and 
    #Y-arrays assuming same parameters are chosen.

    brightness = np.zeros(0)
    for i in range(len(camera_numbers)):
        camera_number = camera_numbers[i]
        
        img_fn = periodic_image_finder.find_periodic_images_full_fn(camera_number, timestamp, timestamp, 1)
        if clipped:
            img = flat_field.get_final_cleaned_image_aurora(img_fn[0], reflection_window = reflection_window)[:,:clipped]
        else:
            img = flat_field.get_final_cleaned_image_aurora(img_fn[0], reflection_window = reflection_window)

        flat_img = binning.bucket(img, (bin_factor, bin_factor)).flatten()

        brightness = np.hstack((brightness, flat_img))
    
    return brightness

def get_x_y_brightness(timestamp, camera_numbers=[4,5,6,7], bin_factor = 8, alignment='north', translate = False, reference_time=1531350000, reflection_window = 600, clipped=False, projection='old'):
    #Returns X, Y and brightness arrays.
    if translate and reference_time == 1531350000:
        print('Warning: in order to account for gondola translation, current and reference time must be specified')

    brightness = get_brightness_array(timestamp, camera_numbers, bin_factor, reflection_window, clipped)
    X, Y = get_x_y_arrays(camera_numbers, bin_factor, alignment, translate, timestamp, reference_time, clipped, projection=projection)
    return X, Y, brightness

def plot_projection(x, y, b, vmin=None, vmax=None, fn='fig.png', savefig=True, limits = [(0, 10), (0, 10)], bin_factor = 4,
                    outpath='/home/christopher/PMC-Analysis/skywinder_analysis/resources/temp/', colormap=cm.inferno):
    
    extent = max(np.abs(limits[0][1] - limits[0][0]),
                 np.abs(limits[1][1] - limits[1][0]))
    n_pixels = extent*64./bin_factor
    scaling = 250.*(50/n_pixels)**2
    size = (x**2 + y**2 + 45**2)/60**2

    fig = plt.figure(figsize=(20,20))
    ax = fig.add_subplot(111)
    
    if not vmin:
        vmin = ss.scoreatpercentile(b, 0.1)
    if not vmax:
        vmax = ss.scoreatpercentile(b, 99.9)
        
    scat = ax.scatter(x, y, s=size*scaling, c=b, cmap=colormap, vmin=vmin, vmax=vmax,
                  alpha=.9)
    ax.set_aspect('equal')
    ax.set_xlabel('x (km)')
    ax.set_ylabel('y (km)')
    ax.set_xlim(limits[0])
    ax.set_ylim(limits[1])
    if savefig:
        fig.savefig(os.path.join(outpath, fn), dpi=100)
    fig.clf()
    plt.close(fig)
        
def get_vmin_vmax(start, end, camera_numbers=[6,7], reflection_window = 600, percentiles = [0.1, 99.9]):
    #Returns vmin and vmax values for image sequence
    fns = []
    for cam in camera_numbers:
        fn_list = periodic_image_finder.find_periodic_images_full_fn(cam, start, end, 600)
        for fn in fn_list:
            fns.append(fn)
            
    vmin = []
    vmax = []

    for i in range(len(fns)):
        img = flat_field.get_final_cleaned_image(fns[i], reflection_window)
        vmin.append(ss.scoreatpercentile(img, percentiles[0]))
        vmax.append(ss.scoreatpercentile(img, percentiles[1]))

    return min(vmin), max(vmax)








def to_years(num_seconds):
    return to_days(num_seconds) / 365.25


def from_years(num_years):
    return from_days(num_years * 365.25)


def to_days(num_seconds):
    return to_hours(num_seconds) / 24.0


def from_days(num_days):
    return from_hours(num_days * 24.0)


def to_julday(num_seconds):
    return (num_seconds / 86400.0 + 2440587.5)


def from_julday(jd):
    return ((jd - 2440587.5) * 86400.0)


def to_hours(num_seconds):
    return to_minutes(num_seconds) / 60.0


def from_hours(num_hours):
    return from_minutes(num_hours * 60.0)


def to_minutes(num_seconds):
    return num_seconds / 60.0


def from_minutes(num_minutes):
    return num_minutes * 60.0


def to_seconds(num_seconds):
    return num_seconds


def from_seconds(num_seconds):
    return num_seconds


def to_milliseconds(num_seconds):
    return num_seconds * 1000.0


def from_milliseconds(num_milliseconds):
    return num_milliseconds / 1000.0


def time_string(num_seconds):
    if num_seconds > from_years(999.0):
        return "%.0e yr" % to_years(num_seconds)
    elif num_seconds > from_years(1.0):
        return "%.1f yr" % to_years(num_seconds)
    elif num_seconds > from_days(1.0):
        return "%.1f dy" % to_days(num_seconds)
    elif num_seconds > from_hours(1.0):
        return "%.1f hr" % to_hours(num_seconds)
    elif num_seconds > from_minutes(1.0):
        return "%.1f min" % to_minutes(num_seconds)
    elif num_seconds > from_seconds(10.0):
        return "%.1f s" % to_seconds(num_seconds)
    elif num_seconds > from_seconds(1.0):
        return "%.2f s" % to_seconds(num_seconds)
    else:
        return "%.0f ms" % to_milliseconds(num_seconds)



import os
import sys
import time

import matplotlib.animation as animation
import matplotlib.cm as cm
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats
from skywinder_analysis.lib.tools import blosc_file
from skywinder_analysis.lib.image_processing import binning, rolling_flat_field, flat_field, stitching
from skywinder_analysis.lib.skywinder_analysis_app import skywinder_analysis_app


class StitchedRollingMovieMakerApp(skywinder_analysis_app.App):
    def ani_frame_from_raw(self):
        self.create_output()
        self.logger.info('Starting animation of frames')
        imd = np.zeros(self.settings.raw_image_size)

        frames = min([len(filenames) for filenames in self.settings.all_filenames])
        for filenames in self.settings.all_filenames:
            self.logger.info('%d filenames passed' % len(filenames))
        fig = plt.figure(frameon=False)
        ax = fig.add_subplot(111)
        ax.get_xaxis().set_visible(False)
        ax.get_yaxis().set_visible(False)
        fig.subplots_adjust(left=0, bottom=0, right=1, top=1, wspace=None, hspace=None)

        if self.settings.bin_pixels:
            img = stitching.build_stitched_image(4 * [binning.bucket(imd, self.settings.bin_pixels)])
        else:
            img = stitching.build_stitched_image(4 * [imd])
        vsize = img.shape[0] / self.settings.dpi
        hsize = img.shape[1] / self.settings.dpi
        fig.set_size_inches([hsize, vsize])

        im = ax.imshow(img, cmap=self.settings.color_scheme, interpolation='nearest')

        xfig, yfig = fig.get_dpi() * fig.get_size_inches()
        fontsize = yfig / 32
        txt0 = ax.text(0.01 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color='black', fontsize=fontsize)
        txt1 = ax.text(0.26 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color='black', fontsize=fontsize)
        txt2 = ax.text(0.51 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color='black', fontsize=fontsize)
        txt3 = ax.text(0.76 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color='black', fontsize=fontsize)
        time_txt = ax.text(0.01 * xfig, 0.05 * yfig, 'time:', ha='left', va='top', color='black', fontsize=fontsize)
        all_txt = [txt0, txt1, txt2, txt3]
        # plt.tight_layout()
        start_at = time.time()
        ax = plt.gca()

        flat_fields = []
        for j, filenames in enumerate(self.settings.all_filenames):
            self.logger.info('Creating ff %d of %d' % (j + 1, len(self.settings.all_filenames)))
            rolling_ff = rolling_flat_field.RollingFlatField(size=self.settings.rolling_flat_field_size)
            for i in range(self.settings.rolling_flat_field_size // 2):
                rolling_ff.roll(self.settings.all_filenames[j][i])
            flat_fields.append(rolling_ff)

        # Populate rolling flat fields with first size/2 images.

        def update_img(n):
            elapsed = time.time() - start_at
            if n:
                time_per_frame = elapsed / n
            else:
                time_per_frame = 1
            sys.stdout.flush()

            imgs = []
            for camera, txt, ff, filenames in zip(self.settings.cameras, all_txt, flat_fields,
                                                  self.settings.all_filenames):
                try:
                    if n > (len(filenames) - (self.settings.rolling_flat_field_size / 2)):
                        pass
                    else:
                        m = n + (self.settings.rolling_flat_field_size // 2)
                        ff.roll(filenames[m])
                        self.logger.info('Rolling to index %d for ff on image %d' % (m, n))
                    flat_field_image = ff.generate_flat_field()
                    img, _ = blosc_file.load_blosc_image(filenames[n])
                    img = flat_field.apply_flat_field(img, flat_field_image)

                    try:
                        time_string = filenames[n].split('/')[-1].split('_')[1]
                        time_string = time_string[0:2] + ':' + time_string[2:4] + ':' + time_string[4:]
                        date_string = filenames[n].split('/')[-1].split('_')[0]
                        time_string = 'Camera ' + str(camera) + ' ' + date_string + ' ' + time_string
                        txt.set_text(time_string)
                    except Exception:
                        pass

                except Exception as e:
                    self.logger.info(e)
                    return

                if self.settings.bin_pixels:
                    img = binning.bucket(img, self.settings.bin_pixels)

                imgs.append(img)

            means = [np.mean(img[self.settings.mean_region[0]:self.settings.mean_region[1],
                             self.settings.mean_region[2]:self.settings.mean_region[3],
                             ]) for img in imgs]
            mean_mean = np.mean(means)
            releveled_imgs = []
            for mean, img in zip(means, imgs):
                self.logger.info('Releveling. Mean of means is %.1f, mean of img is %.1f.' % (mean_mean, mean))
                releveled_imgs.append((mean_mean / mean) * img)

            stitched_img = stitching.build_stitched_image(releveled_imgs)
            im.set_data(stitched_img)

            vmin = scipy.stats.scoreatpercentile(
                stitched_img[self.settings.level_region[0]:self.settings.level_region[1],
                self.settings.level_region[2]:self.settings.level_region[3]],
                self.settings.min_percentile)
            vmax = scipy.stats.scoreatpercentile(
                stitched_img[self.settings.level_region[0]:self.settings.level_region[1],
                self.settings.level_region[2]:self.settings.level_region[3]],
                self.settings.max_percentile)
            self.logger.info('%.1f percentile is %.1f, %.1f percentile is %.1f' % (self.settings.min_percentile, vmin, self.settings.max_percentile, vmax))
            #vmin = scipy.stats.scoreatpercentile(stitched_img, 0.1)
            #vmax = scipy.stats.scoreatpercentile(stitched_img, 99.9)
            im.set_clim(vmin, vmax)

            self.logger.info("\r%d of %d %.1f minutes elapsed, %.1f minutes remaining" % (
                n, frames, elapsed / 60,
                (frames - n) * time_per_frame / 60))

            return im, txt0, txt1, txt2, txt3

        ani = animation.FuncAnimation(fig, update_img, frames, interval=100)

        ani_fn = os.path.join(self.out_path, self.settings.output_name)
        ani.save(ani_fn, writer=self.settings.writer, dpi=self.settings.dpi)
        return ani


if __name__ == "__main__":
    app = StitchedRollingMovieMakerApp()
    app.ani_frame_from_raw()
    app.end()



import os
import numpy as np
import pandas as pd

pointing_dir = '/data/mounted_filesystems/nas2/resources/pointing_solutions/'

def get_camera_pointing_array(camera_number):
    if camera_number != 99:
        fn = 'c{0}_2018-07-12_2330_solution.csv'.format(camera_number)
    else:
        fn = 'c99_2019-12-18_0205_solution.csv'.format(camera_number)
    df = pd.read_csv(os.path.join(pointing_dir, fn))
    az = np.radians(np.asarray(df.az))
    if camera_number != 99:
        az += np.radians(81.6525)
    az = np.reshape(az, (5,5)) 
    alt = np.reshape(np.radians(np.asarray(df.alt)), (5,5))
    return az, alt



from skywinder_analysis.lib.tools import generic
import matplotlib.animation as animation

settings = generic.Class()

settings.section = None
settings.bin_pixels = (4, 4)
settings.dpi = 100
settings.rolling_flat_field_size = 20

settings.camera_number = 0

settings.raw_image_size = (3232, 4864)
settings.output_name = 'movie.mp4'
settings.filenames = []  # use glob to make list of filenames
settings.writer = animation.writers['ffmpeg'](fps=10, codec='h264', bitrate=2**20)
settings.level_region = [0//settings.bin_pixels[0],3232//settings.bin_pixels[0],0//settings.bin_pixels[1],4864//settings.bin_pixels[1]]



from skywinder_analysis.lib.skywinder_analysis_app import skywinder_analysis_app
import glob
import os
import datetime
import scipy.stats
import scipy.signal
import scipy.interpolate
import pandas as pd
from skywinder_analysis.lib.image_processing import flat_field, binning
from skywinder_analysis.lib.pointing import blob_tools, blobs
from skywinder_turbo.camera.image_processing import hot_pixels
import matplotlib.pyplot as plt
import numpy as np


class img_container_series():
    def __init__(self, cam_num=0):
        self.cam_num = cam_num
        self.ff = False
        self.imgs = []
        self.fh = False
        self.fw = False
        self.blobs = []
        self.timestamps_ms = []


class blob_trajectory():
    def __init__(self, x, y, timestamp, f_az, f_alt, update_index):
        self.x = []
        self.y = []
        self.alt = []
        self.az = []
        self.delta_x = []
        self.delta_y = []
        self.delta_alt = []
        self.delta_az = []
        self.last_update = 0
        self.length = 0
        self.x.append(x)
        self.y.append(y)
        alt = f_alt(x,y)[0]
        az = f_az(x,y)[0]
        self.alt.append(alt)
        self.az.append(az)
        self.timestamps = []
        self.timestamps.append(timestamp)
        self.delta_x.append(0)
        self.delta_y.append(0)
        self.delta_alt.append(0)
        self.delta_az.append(0)
        self.last_update = update_index
        self.length += 1
        
    def update(self, x, y, timestamp, f_az, f_alt, update_index):
        self.delta_x.append(x - self.x[-1])
        self.delta_y.append(y - self.y[-1])
        alt = f_alt(x,y)[0]
        az = f_az(x,y)[0]
        self.delta_alt.append(alt - self.alt[-1])
        self.delta_az.append(az - self.az[-1])
        self.x.append(x)
        self.y.append(y)
        self.alt.append(alt)
        self.az.append(az)
        self.timestamps.append(timestamp)
        self.last_x = x
        self.last_y = y
        self.last_update = update_index
        self.length += 1


def group_blob_trajectories(image_container, f_az, f_alt, min_distance=10):
    blob_trajectories = []
    for n in range(len(image_container.blobs)):
        if n == 0:
            # Initialize
            df = image_container.blobs[n]
            ts = image_container.timestamps_ms[n]
            for i in range(len(df)):
                blob_trajectories.append(blob_trajectory(df.iloc[i].x, df.iloc[i].y, ts, f_az, f_alt, n))
        else:
            df = image_container.blobs[n]
            ts = image_container.timestamps_ms[n]
            for b_t in blob_trajectories:
                distance = np.sqrt(np.square(b_t.x[-1] - df.x) + np.square(b_t.y[-1] - df.y))
                if distance.min() < min_distance:
                    closest = df.iloc[distance.argsort()[0]]
                    b_t.update(closest.x, closest.y, ts, f_az, f_alt, n)
    return blob_trajectories    


class LSPeriodogramApp(skywinder_analysis_app.App):
    def get_periodograms(self):
        self.create_output()
        self.logger.info('Creating container classes')
        for fns in self.settings.all_filename_lists:
            ic = img_container_series(cam_num=self.settings.camera_number)
            for fn in fns:
                img = flat_field.get_final_cleaned_image(fn, self.settings.new_flat_field_window)
                ic.imgs.append(img)
                # Get blobs
                myblobs = self.get_blobs_from_img(img, 4, show_plot=False,blob_threshold=8,
                                             kernel_size=8, kernel_sigma=2,fitting_region_size=16,
                                             fit_blobs=True, cell_size=32, max_blobs_per_cell=2, rmax=4,
                                             low_sigma_filter=None)
                # Switch blobs to pandas for convenience
                ic.blobs.append(pd.DataFrame([vars(b) for b in myblobs]))
                ic.timestamps_ms.append(int(fn[-19:-6])) # get timestamp in ms from filename
            df = pd.read_csv(self.settings.pointing_solutions_dir)
            f_alt = scipy.interpolate.interp2d(df.h, df.w, df.alt, kind='cubic', bounds_error=False, fill_value=np.nan)
            f_az = scipy.interpolate.interp2d(df.h, df.w, df.az, kind='cubic', bounds_error=False, fill_value=np.nan)
            blob_trajectories = group_blob_trajectories(ic, f_az, f_alt)
            followed_stars = [bt for bt in blob_trajectories if bt.length >= self.settings.min_blob_trajectory_length]
            self.logger.info('%d stars followed.' % len(followed_stars))
            if len(followed_stars) == 0:
                self.logger.info('No stars sufficiently followed.')
            else:
                f = np.linspace(0.01, 2,1000)
                alt_periodograms = []
                az_periodograms = []
                fig, axs = plt.subplots(2, 1, figsize=(9,9))
                _axs = axs.flatten()
                for followed_star in followed_stars:
                    y = np.array(followed_star.alt)
                    t = (np.array(followed_star.timestamps)-followed_star.timestamps[0])/1000.
                    linear_fit = np.polyfit(t, y, 1)
                    fit = linear_fit[0]*t + linear_fit[1]
                    pgram = scipy.signal.lombscargle(t, y-fit, f, normalize=True)
                    alt_periodograms.append(pgram)

                    y = np.array(followed_star.az)
                    linear_fit = np.polyfit(t, y, 1)
                    fit = linear_fit[0]*t + linear_fit[1]
                    pgram = scipy.signal.lombscargle(t, y-fit, f, normalize=True)
                    az_periodograms.append(pgram)
                _axs[0].errorbar(f/(2.0*np.pi),
                            np.mean(np.stack(alt_periodograms), axis=0),
                            np.std(np.stack(alt_periodograms), axis=0),
                            fmt='-', color='blue', ecolor='black', capsize=0,
                           )
                _axs[0].set_title('Alt')
                _axs[1].errorbar(f/(2.0*np.pi),
                            np.mean(np.stack(az_periodograms), axis=0),
                            np.std(np.stack(az_periodograms), axis=0),
                            fmt='-', color='blue', ecolor='black', capsize=0,
                           )
                _axs[1].set_title('Az')
                for ax in axs:
                    ax.grid(True)
                    ax.set_xlim(0, .25)
                    ax.set_ylim(0)
                    ax.set_xlabel('frequency (Hz)')
                    ax.set_ylabel('Lomb-Scargle Power')
                fn_ts = fns[0].split('/')[-1].split('=')[-1][:10]
                ts_utc = datetime.datetime.utcfromtimestamp(int(fn_ts))
                ts_string = ts_utc.strftime('%Y-%m-%d_%H:%M')
                title = ts_string + ('+%d seconds' % self.settings.duration) + (' %d stars followed'%len(followed_stars))
                fig.suptitle(title)
                fig_fn = os.path.join(self.out_path, '%s.png'%ts_string)
                plt.savefig(fig_fn)
        return alt_periodograms, az_periodograms

    def get_blobs_from_img(self, img, cam_number, show_plot=True,
                           blob_threshold=6, kernel_size=8, kernel_sigma=2,
                           fitting_region_size=32, fit_blobs=True,cell_size=128,
                           max_blobs_per_cell=1, rmax=4,
                           low_sigma_filter=0.5, high_sigma_filter=10):
        hpm = hot_pixels.HotPixelMasker(np.load(self.settings.hot_pixel_dirs[cam_number]), self.settings.image_shape)
        img = hpm.process(img)
        b = blobs.BlobFinder(img, blob_threshold, kernel_size, kernel_sigma,
                             fitting_region_size, fit_blobs,cell_size,
                             max_blobs_per_cell, rmax)
        my_blobs_list = b.blobs
        if not low_sigma_filter:
            good_blobs = my_blobs_list
        else:
            good_blobs = [blob for blob in my_blobs_list if blob.sigma > low_sigma_filter]
            good_blobs = [blob for blob in good_blobs if blob.sigma < high_sigma_filter]
        if show_plot:
            show_blobs(img, good_blobs)
        self.logger.info('Found %d good blobs.' % len(good_blobs))
        return good_blobs


if __name__ == "__main__":
    import warnings; warnings.simplefilter('ignore')
    app = LSPeriodogramApp()
    app.get_periodograms()



import os
import numpy as np
from matplotlib import pyplot as plt
import matplotlib.cm as cm
import scipy.stats as ss
from scipy.interpolate import RectBivariateSpline as RBS
from skywinder_analysis.lib.image_processing import flat_field_piggyback, binning
from skywinder_analysis.lib.image_projection import pixel_projection as pp
from skywinder_analysis.lib.tools import GPS_pointing_and_sun as GPS
from skywinder_analysis.lib.tools import periodic_image_finder
from astropy.coordinates import EarthLocation
from astropy import units as u


def get_x_y_arrays(bin_factor = 8, alignment='north', translate = False, current_time=1577000000, reference_time=1577000000, clipped=False, masked=True):
    #Gets X- and Y- arrays for scatter plot. alignment='north' returns arrays in coordinate system with +y to the north, +x to east.
    #alignment='antisun' returns arrays in coordinate system with +y in antisun direction.

    #In order to compensate for gondola translation, current time and reference time must be specified. The origin of the coordinate system will be 
    #directly overhead at the reference time - other images will be translated relative to that point. This only works for alignment='north'.

    x_old = np.arange(0, 3232, 34)
    y_old = np.arange(0, 4864, 34)

    if clipped:
        x_new = np.arange(0, 3232, bin_factor)
        y_new = np.arange(0, clipped, bin_factor)
    else:
        x_new = np.arange(0, 3232, bin_factor)
        y_new = np.arange(0, 4864, bin_factor)

      
    fn = '/data/mounted_filesystems/nas2/resources/pixel_coordinates_downsampled/c99_coordinates.csv'
    XY = np.loadtxt(fn, delimiter=',')
    X = XY.T[0].reshape(96,144)
    Y = XY.T[1].reshape(96,144)
    f_x = RBS(x_old, y_old, X)
    X_inter = f_x(x_new, y_new)
    f_y = RBS(x_old, y_old, Y)
    Y_inter = f_y(x_new, y_new)
    if masked:
        MASK, indices = flat_field_piggyback.get_mask(bin_factor)    
        X_array = X_inter[indices].flatten()
        Y_array = Y_inter[indices].flatten()
    else:
        X_array = X_inter.flatten()
        Y_array = Y_inter.flatten()
    
    if alignment == 'north':
        az_offset = GPS.get_pointing_rotator(current_time)

        if translate:
            X_offset = pp.get_image_translation_Earth_frame(reference_time, current_time)
        else:
            X_offset=np.array([0, 0, 0])

        scaling = 1.0 + X_offset[2]/45.0
        
        X_array_trans = (X_array*np.cos(az_offset)*scaling
                        + Y_array*np.sin(az_offset)*scaling
                        + X_offset[0])
        Y_array_trans = (-X_array*np.sin(az_offset)*scaling
                        + Y_array*np.cos(az_offset)*scaling
                        + X_offset[1])

    elif alignment == 'antisun':        
        X_array_trans = X_array
        Y_array_trans = Y_array
    else:
        print('alignment must be either \'north\' or \'antisun\'')
        return

    return X_array_trans, Y_array_trans

def convert_pixel_arrays_to_sky_coordinates(X_px, Y_px, timestamps, bin_factor = 8, reference_time=1577000000):
    #Converts arrays of pixel coordinates to coordinates on the cloud layer. Returns arrays in coordinate system with +y to the north, +x to east.
    #In order to compensate for gondola translation, current time and reference time must be specified. The origin of the coordinate system will be 
    #directly overhead at the reference time - other images will be translated relative to that point. 

    x_old = np.arange(0, 3232, 34)
    y_old = np.arange(0, 4864, 34)
    fn = '/data/mounted_filesystems/nas2/resources/pixel_coordinates_downsampled/c99_coordinates.csv'
    XY = np.loadtxt(fn, delimiter=',')
    X = XY.T[0].reshape(96,144)
    Y = XY.T[1].reshape(96,144)
    f_x = RBS(x_old, y_old, X)
    f_y = RBS(x_old, y_old, Y)
    X_array = np.zeros(0)
    Y_array = np.zeros(0)
    for x, y, timestamp in zip(X_px, Y_px, timestamps):
        X0 = f_x(x*bin_factor + bin_factor/2., y*bin_factor + bin_factor/2.).flatten()
        Y0 = f_y(x*bin_factor + bin_factor/2., y*bin_factor + bin_factor/2.).flatten()
    
        az_offset = GPS.get_pointing_rotator(timestamp)
        

        X_offset = pp.get_image_translation_Earth_frame(reference_time, timestamp)
        
        scaling = 1.0 + X_offset[2]/45.0

        X0_trans = (X0*np.cos(az_offset)*scaling
                        + Y0*np.sin(az_offset)*scaling
                        + X_offset[0])
        Y0_trans = (-X0*np.sin(az_offset)*scaling
                        + Y0*np.cos(az_offset)*scaling
                        + X_offset[1])
        X_array = np.hstack((X_array, X0_trans))
        Y_array = np.hstack((Y_array, Y0_trans))    


    return X_array, Y_array

def convert_pixel_arrays_to_sky_coordinates_array(X_px, Y_px, timestamps, bin_factor = 8, reference_time=1577000000):
    #Converts arrays of pixel coordinates to coordinates on the cloud layer. Returns arrays in coordinate system with +y to the north, +x to east.
    #In order to compensate for gondola translation, current time and reference time must be specified. The origin of the coordinate system will be 
    #directly overhead at the reference time - other images will be translated relative to that point. 

    x_old = np.arange(0, 3232, 34)
    y_old = np.arange(0, 4864, 34)
    fn = '/data/mounted_filesystems/nas2/resources/pixel_coordinates_downsampled/c99_coordinates.csv'
    
    XY = np.loadtxt(fn, delimiter=',')
    X = XY.T[0].reshape(96,144)
    Y = XY.T[1].reshape(96,144)
    f_x = RBS(x_old, y_old, X)
    f_y = RBS(x_old, y_old, Y)
    X_array = np.zeros(0)
    Y_array = np.zeros(0)
    #for x, y, timestamp in zip(X_px, Y_px, timestamps):
        
    X0 = f_x.ev(X_px*bin_factor + bin_factor/2., Y_px*bin_factor + bin_factor/2.).flatten()
    Y0 = f_y.ev(X_px*bin_factor + bin_factor/2., Y_px*bin_factor + bin_factor/2.).flatten()
    
    az_offset = GPS.get_pointing_rotator_array(timestamps)
    X_offset = pp.get_image_translation_Earth_frame_array(reference_time, timestamps)

    scaling = 1.0 + X_offset[2,:]/45.0

    X0_trans = (X0*np.cos(az_offset)*scaling
                    + Y0*np.sin(az_offset)*scaling
                    + X_offset[0,:])
    Y0_trans = (-X0*np.sin(az_offset)*scaling
                    + Y0*np.cos(az_offset)*scaling
                    + X_offset[1,:])

    return X0_trans, Y0_trans

def get_brightness_array(timestamp, bin_factor = 8, reflection_window = 600, clipped=False, masked=True):
    #Returns brightness array for given time, cameras, etc. Output will have same format as X- and 
    #Y-arrays assuming same parameters are chosen.
 
    img_fn = periodic_image_finder.find_periodic_images_full_fn(99, timestamp, timestamp, 1)
    
    if clipped:
        img = flat_field_piggyback.get_final_cleaned_image(img_fn[0], reflection_window = reflection_window, masked=masked)[:,:clipped]
    else:
        img = flat_field_piggyback.get_final_cleaned_image(img_fn[0], reflection_window = reflection_window, masked=masked)
    
    if masked:
        MASK, indices = flat_field_piggyback.get_mask(bin_factor)
        flat_img = binning.bucket(img, (bin_factor, bin_factor))[indices].flatten()
    else:
        flat_img = binning.bucket(img, (bin_factor, bin_factor)).flatten()
    
    return flat_img

def get_x_y_brightness(timestamp, bin_factor = 8, alignment='north', translate = False, reference_time=1577000000, reflection_window = 600, clipped=False, masked=True):
    #Returns X, Y and brightness arrays.
    brightness = get_brightness_array(timestamp, bin_factor, reflection_window, clipped, masked)
    X, Y = get_x_y_arrays(bin_factor, alignment, translate, timestamp, reference_time, clipped, masked)
    return X, Y, brightness

def plot_projection(x, y, b, vmin=None, vmax=None, fn='fig.png', savefig=True, limits = [(0, 10), (0, 10)], bin_factor = 4,
                    outpath='/home/christopher/PMC-Analysis/skywinder_analysis/resources/temp/', colormap=cm.inferno):
    
    extent = max(np.abs(limits[0][1] - limits[0][0]),
                 np.abs(limits[1][1] - limits[1][0]))
    n_pixels = extent*64./bin_factor
    scaling = 250.*(50/n_pixels)**2
    size = (x**2 + y**2 + 45**2)/60**2

    fig = plt.figure(figsize=(20,20))
    ax = fig.add_subplot(111)
    
    if not vmin:
        vmin = ss.scoreatpercentile(b, 0.1)
    if not vmax:
        vmax = ss.scoreatpercentile(b, 99.9)
        
    scat = ax.scatter(x, y, s=size*scaling, c=b, cmap=colormap, vmin=vmin, vmax=vmax,
                  alpha=.9)
    ax.set_aspect('equal')
    ax.set_xlabel('x (km)')
    ax.set_ylabel('y (km)')
    ax.set_xlim(limits[0])
    ax.set_ylim(limits[1])
    if savefig:
        fig.savefig(os.path.join(outpath, fn), dpi=100)
    fig.clf()
    plt.close(fig)
        
def get_vmin_vmax(start, end, reflection_window = 600, percentiles = [0.1, 99.9], masked=True):
    #Returns vmin and vmax values for image sequence
    fns = []
    fn_list = periodic_image_finder.find_periodic_images_full_fn(99, start, end, 600)
    for fn in fn_list:
        fns.append(fn)
            
    vmin = []
    vmax = []

    for i in range(len(fns)):
        img = flat_field_piggyback.get_final_cleaned_image(fns[i], reflection_window, masked)
        vmin.append(ss.scoreatpercentile(img, percentiles[0]))
        vmax.append(ss.scoreatpercentile(img, percentiles[1]))

    return min(vmin), max(vmax)








from default_settings import settings
import matplotlib.animation as animation
import os
from pmc_analysis.lib.tools import periodic_image_finder
import matplotlib.cm as cm
import datetime
import calendar

n = 1531402200 # 2018-07-12_1330
start = n - (3*60)
stop = start + (8*60)
interval = 2 

c4_filenames = periodic_image_finder.find_periodic_images_full_fn(4, start, stop, interval)
settings.all_filenames = [c4_filenames]
settings.cameras = [4]

settings.use_new_flat_field = True
settings.new_flat_field_reflection_window = 600


settings.bin_pixels = (8,8)

#settings.level_region = [4864 // (4 * settings.bin_pixels[0]),
#                         (4864) // (settings.bin_pixels[0]),#(3 * 4864) // (4 * settings.bin_pixels[0]),
#                         0 // settings.bin_pixels[1],
#                         (len(settings.all_filenames) * 3232) // settings.bin_pixels[1]]
# Level full stitched by central part of all cameras
settings.level_region = [3232//(4 * settings.bin_pixels[0]), (3 * 3232)//(4 * settings.bin_pixels[0]), 0//settings.bin_pixels[1], 4864//settings.bin_pixels[1]]
# Level single camera by central part



settings.color_scheme = cm.Blues_r
#settings.fontcolor = 'white'
#settings.crop = [3232//4,3232,4864//4,5*4864//8]
settings.colorscale_by_frame = True
settings.vmin = False
settings.vmax = False

settings.individual_frames = False
#settings.split_video_len = (20*60 // interval)
settings.font_proportion = 1./48.
settings.no_text = False



import numpy as np

image_dimensions = (3232, 4864)

frame_info_dtype = np.dtype([('frame_id', np.uint64), ('timestamp', np.uint64),
                             ('frame_status', np.uint32), ('is_filled', np.uint32)])

chunk_dtype = np.dtype([('image_chunk_identifier', '<u4'),  # NB: This is part of the GigEVision chunk header, so order
                        ('image_chunk_length', '<u4'),  # is little endian instead of big endian
                        ('acquisition_count', '>u4'),
                        ('lens_status_focus', '>u2'),
                        ('lens_aperture', 'u1'),
                        ('lens_focal_length', 'u1'),
                        ('exposure_us', '>u4'),
                        ('gain_db', '>u4'),
                        ('sync_in', '>u2'),
                        ('sync_out', '>u2'),
                        ('reserved_1', '>u4'),
                        ('reserved_2', '>u4'),
                        ('reserved_3', '>u4'),
                        ('reserved_4', '>u4'),
                        ('reserved_5', '>u4'),
                        ('chunk_identifier', '<u4'),  # NB: This is part of the GigEVision chunk header, so order
                        ('chunk_length', '<u4')])  # is little endian instead of big endian

chunk_num_bytes = chunk_dtype.itemsize


def decode_aperture_chunk_data(chunk_data_aperture_byte):
    """
    Convert value provided in byte 7 of chunk data to FStop number

    Formula provided on page 100 of GigE_Features_Reference.pdf (v5.3.2)
    https://www.alliedvision.com/fileadmin/content/documents/products/cameras/various/features/GigE_Features_Reference.pdf

    Parameters
    ----------
    chunk_data_aperture_byte: int 0-255
        value of chunk data byte 7, called chunk_dtype['lens_aperture']

    Returns
    -------
    fstop: float

    """
    return 2 ** ((chunk_data_aperture_byte - 8) / 16.0)


_lens_status_error_message_strings = {0: "No error",
                                      1: "Lens failed query by camera",
                                      2: "Lens communication error 2 (can occur when removing lens)",
                                      3: "Lens communication error 3 (can occur when removing lens)",
                                      4: "Lens remained busy for longer than 10 seconds",
                                      5: "Lens focus zero stop not detected",
                                      6: "Lens focus infinity stop not detected"}


def decode_lens_status_chunk_data(status_bits):
    """

    Parameters
    ----------
    chunk_data_lens_status_byte : int 0-63
        value of top 6 bits of chunk data byte 5, stored in image index as 'lens_status' and equal to chunk_dtype['lens_status_focus']>>10

    Returns
    -------
    status dictionary with keys 'error', 'lens_attached', 'auto_focus', 'last_error', 'last_error_message'

    """
    last_error = (status_bits & 0x7)
    last_error_message = _lens_status_error_message_strings.get(last_error, ("Unexpected eror code %d" % last_error))
    result = dict(error=bool(status_bits & 0x20),
                  lens_attached=bool(status_bits & 0x10),
                  auto_focus=bool(status_bits & 0x08),
                  last_error=last_error,
                  last_error_message=last_error_message)
    return result



import re


def from_camel_to_lower(name):
    '''
    Copied from: http://stackoverflow.com/a/1176023/721964
    '''
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()



from skywinder_analysis.lib.tools import generic
import matplotlib.animation as animation

settings = generic.Class()

settings.section = None
settings.bin_pixels = (4, 4)
settings.dpi = 100
settings.saved_flat_field_path = None
settings.flat_field_filenames = None

settings.raw_image_size = (3232, 4864)
settings.output_name = 'movie.mp4'
settings.filenames = []  # use glob to make list of filenames
settings.writer = animation.writers['ffmpeg'](fps=20, codec='h264', bitrate=2 ** 20)
settings.camera_number = 0
settings.rotate = False
settings.min_percentile = 0.1
settings.max_percentile = 99.9

if settings.rotate:
    settings.level_region = [4864 // (4 * settings.bin_pixels[0]),
                             4864 // (settings.bin_pixels[0]),
                             0 // settings.bin_pixels[1],
                             3232 // settings.bin_pixels[1]]
else:
    settings.level_region = [0 // settings.bin_pixels[0],
                             3232 // settings.bin_pixels[0],
                             4864 // (4 * settings.bin_pixels[1]),
                             4864 // (settings.bin_pixels[1])]
settings.gaussian_filter_sigma = False



from __future__ import division

import numpy as np
from scipy.interpolate import interp1d
from scipy import optimize as opt
import os
import datetime as dt

from skywinder_analysis.lib.tools import blosc_file, check_c5_focus
from skywinder_analysis.lib.image_processing import sky_brightness_model, dark_images, binning


def get_final_cleaned_image(fn, reflection_window=600, masked=True):
    #Reflection window should be given in seconds. The actual window used is the 
    #value given rounded down to the nearest 10 minutes (or rounded up to 10 minutes if less than 10 minutes)
    #Returns final cleaned image in units of counts/second, with some arbitrary offset.
    timestamp = int(fn.split('=')[-1][:10])
    
    calibrated_image = get_calibrated_image(fn)
    reflection_image = get_composite_flat(timestamp, reflection_window)
    if masked:
        MASK, indices = get_mask()
        return 10*(calibrated_image - reflection_image)*MASK
    else:
        return 10*(calibrated_image - reflection_image)

def get_mask(bin_factor=1):
    MASK = np.ones((3232,4864))
    for i in range(500):
        MASK[i, :4*(500 - i)] = np.nan
        MASK[-i-1, -4*(500 - i):] = np.nan
    mask = np.where(np.isnan(MASK) == False)
    binned_MASK = binning.bucket(MASK, (bin_factor,bin_factor))
    binned_mask = np.where(np.isnan(binned_MASK) == False)
    return binned_MASK, binned_mask

def get_calibrated_image(fn):
    camera_number = 99
    timestamp = int(fn.split('=')[-1][:10])
    
    img, chunk = blosc_file.load_blosc_image(fn)
    exposure = float(chunk[0][6])
    
    flat = get_flat_field_image_pre_flight(camera_number)
    dark = dark_images.get_dark_image(camera_number, exposure)
    
    calibrated_image = ((img.astype('float') - dark.astype('float'))/flat.astype('float'))*np.mean(flat)*100000/exposure
    return calibrated_image

def get_composite_flat(timestamp, reflection_window = 600):
    N = np.max([reflection_window // 600, 1]) 
    #direc = '/home/christopher/SkyWinder-Analysis/skywinder_analysis/output/foreground_subtracted_composites_piggyback/'
    direc = '/data/mounted_filesystems/nas2/resources/piggyback_moving_averages/'
    fn_list = os.listdir(direc)
    time_list = []
    time0 = 1576454400
    for fn in fn_list:
        date = fn.split('--')[0][-10:]
        month = int(date[:2])
        day = int(date[3:5])
        time = fn.split('--')[1]
        hour = int(time[:2])
        minute = int(time[3:5])
        if month == 12:
            time = time0 + 86400*(day - 16) + 3600*hour + 60*(minute + 5)
        else:
            time = time0 + 86400*(day + 15) + 3600*hour + 60*(minute + 5)
        time_list.append(time)

    times = np.asarray(time_list)
    fns = np.asarray(fn_list)
    sorted_times = times[times.argsort()]
    sorted_fns = fns[times.argsort()] 
    
    index = np.where(sorted_times <= timestamp)[0][-1]
    if N % 2 == 0:
        if timestamp - sorted_times[index] > 300:
            index += 1
            
            
    imd = np.zeros((3232, 4864, 2))
    img = np.zeros((3232, 4864))
    
    for i in range(N - 1):
        temp, _ = blosc_file.load_blosc_image(os.path.join(direc, sorted_fns[index - N // 2 + 1 + i]))
        img += temp.astype('float')/float(N)
    
    imd[:,:,0], _ = blosc_file.load_blosc_image(os.path.join(direc, sorted_fns[index - N // 2]))
        
    x = np.array([sorted_times[index], sorted_times[index + 1]])
    if N % 2 == 0:
        x -= 300
        imd[:,:,1], _ = blosc_file.load_blosc_image(os.path.join(direc, sorted_fns[index + N // 2]))
    else:
        imd[:,:,1], _ = blosc_file.load_blosc_image(os.path.join(direc, sorted_fns[index + N // 2 + 1]))
    
    img_interp = interp1d(x, imd)
    
    img += img_interp(timestamp)/float(N)
    
    img0 = 2*img.astype('float')

    return img0

def get_flat_field_image_pre_flight(camera_number):
    if camera_number == 99:
        camera_number = 7
    flat, _ = blosc_file.load_blosc_image('/data/mounted_filesystems/nas2/resources/flat_fields_pre-flight/camera{0}_flat_field'.format(camera_number))
    return flat
